typedef void (^releaseBytesBlock)(const void* baseAddress);

static void releaseBytesCallback(void* refCon, const void* baseAddress) {
    releaseBytesBlock releaseBytesBlock = refCon;
    releaseBytesBlock(baseAddress);
}

CVReturn CVPixelBufferCreateWithBytes_b(CFAllocatorRef allocator,
                                        size_t width,
                                        size_t height,
                                        OSType pixelFormatType,
                                        void* baseAddress,
                                        size_t bytesPerRow,
                                        void (^releaseBytesBlock)(void* baseAddress),
                                        CFDictionaryRef pixelBufferAttributes,
                                        CVPixelBufferRef* pixelBufferOut) {

    void* releaseRefCon = releaseBytesBlock;
    return CVPixelBufferCreateWithBytes(allocator,
                                        width,
                                        height,
                                        pixelFormatType,
                                        baseAddress,
                                        bytesPerRow,
                                        &releaseBytesCallback,
                                        releaseRefCon,
                                        pixelBufferAttributes,
                                        pixelBufferOut);
}

typedef void (^releasePlanarBytesBlock)(const void* dataPtr, size_t dataSize, size_t numberOfPlanes, const void** planeAddresses);

static void releasePlanarBytesCallback(void*        refCon,
                                       const void*  dataPtr,
                                       size_t       dataSize,
                                       size_t       numberOfPlanes,
                                       const void** planeAddresses) {
    releasePlanarBytesBlock releasePlanarBytesBlock = refCon;
    releasePlanarBytesBlock(dataPtr, dataSize, numberOfPlanes, planeAddresses);
}

CVReturn CVPixelBufferCreateWithPlanarBytes_b(CFAllocatorRef allocator,
                                              size_t width,
                                              size_t height,
                                              OSType pixelFormatType,
                                              void* dataPtr,
                                              size_t dataSize,
                                              size_t numberOfPlanes,
                                              void* planeBaseAddress[],
                                              size_t planeWidth[],
                                              size_t planeHeight[],
                                              size_t planeBytesPerRow[],
                                              void (^releasePlanarBytesBlock)(void* dataPtr, size_t dataSize, size_t numberOfPlanes, void* planeAddresses[]),
                                              CFDictionaryRef pixelBufferAttributes,
                                              CVPixelBufferRef* pixelBufferOut) {
    void* releaseRefCon = releasePlanarBytesBlock;
    return CVPixelBufferCreateWithPlanarBytes(allocator,
                                              width,
                                              height,
                                              pixelFormatType,
                                              dataPtr,
                                              dataSize,
                                              numberOfPlanes,
                                              planeBaseAddress,
                                              planeWidth,
                                              planeHeight,
                                              planeBytesPerRow,
                                              &releasePlanarBytesCallback,
                                              releaseRefCon,
                                              pixelBufferAttributes,
                                              pixelBufferOut);
}

