typedef int (^read_packet)(uint8_t*, int);
typedef int (^write_packet)(uint8_t*, int);
typedef int64_t (^seek)(int64_t, int);

typedef struct {
    read_packet read_packet;
    write_packet write_packet;
    seek seek;
} Callbacks;

static int read_packet_callback(void *opaque, uint8_t *buf, int buf_size)
{
    read_packet read_packet = ((Callbacks*)opaque)->read_packet;
    return read_packet ? read_packet(buf, buf_size) : AVERROR_UNKNOWN;
}

static int write_packet_callback(void *opaque, uint8_t *buf, int buf_size)
{
    write_packet write_packet = ((Callbacks*)opaque)->write_packet;
    return write_packet ? write_packet(buf, buf_size) : AVERROR_UNKNOWN;
}

static int64_t seek_callback(void *opaque, int64_t offset, int whence)
{
    seek seek = ((Callbacks*)opaque)->seek;
    return seek ? seek(offset, whence) : AVERROR_UNKNOWN;
}

AVIOContext *avio_alloc_context_b(
                  unsigned char *buffer,
                  int buffer_size,
                  int write_flag,
                  int (^read_packet)(uint8_t *buf, int buf_size),
                  int (^write_packet)(uint8_t *buf, int buf_size),
                  int64_t (^seek)(int64_t offset, int whence))
{
    Callbacks *callbacks = av_malloc(sizeof(Callbacks));
    callbacks->read_packet = read_packet;
    callbacks->write_packet = write_packet;
    callbacks->seek = seek;
    return avio_alloc_context(buffer,
                  buffer_size,
                  write_flag,
                  callbacks,
                  read_packet ? &read_packet_callback : NULL,
                  write_packet ? &write_packet_callback : NULL,
                  seek ? &seek_callback : NULL);
}

void avio_free_context0(AVIOContext *s)
{
    if(s->opaque)
        av_freep(&s->opaque);

    av_free(s);
}

