constant sampler_t sampler = CLK_NORMALIZED_COORDS_TRUE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_LINEAR;

static float3 yuv_to_rgb(         float3 yuv,
                  constant float* coef,
                           float  lumaZero,
                           float  chromaZero,
                           float  blackLevel) {

  float3 coefR = (float3)(coef[0], coef[1], coef[2]);
  float3 coefG = (float3)(coef[3], coef[4], coef[5]);
  float3 coefB = (float3)(coef[6], coef[7], coef[8]);

  yuv = yuv - (float3)(lumaZero, chromaZero, chromaZero);
  float3 rgb;
  rgb.r = dot(coefR, yuv);
  rgb.g = dot(coefG, yuv);
  rgb.b = dot(coefB, yuv);
  rgb = rgb + (float3)(blackLevel, blackLevel, blackLevel);
  return rgb;
}

static float2 norm(int2 input, read_only image2d_t image) {
  return convert_float2(input) / convert_float2(get_image_dim(image));
}

static float2 imageScale(read_only image2d_t inImage, int2 outDim) {
  float2 inDim    = convert_float2(get_image_dim(inImage));
  float2 outDimF  = convert_float2(outDim);
  return inDim / outDimF;
}

static float3 nv12_nv21_to_rgb(constant  float*    coef,
                                  float     lumaZero,
                                  float     chromaZero,
                                  float     blackLevel,
                                  float     depthScale,
                                  float2    coord,
                        read_only image2d_t inPlane0,
                        read_only image2d_t inPlane1) {

  float3 yuv;
  yuv.x  = read_imagef(inPlane0, sampler, coord).r;
  yuv.yz = read_imagef(inPlane1, sampler, coord).rg;

  return yuv_to_rgb(yuv, coef, lumaZero, chromaZero, blackLevel);
}

static float3 planar_yuv_to_rgb(constant   float*    coef,
                                    float     lumaZero,
                                    float     chromaZero,
                                    float     blackLevel,
                                    float     depthScale,
                                    float2    coord,
                         read_only  image2d_t inPlane0,
                         read_only  image2d_t inPlane1,
                         read_only  image2d_t inPlane2) {

  float3 yuv;
  yuv.x  = read_imagef(inPlane0, sampler, coord).r * depthScale;
  yuv.y  = read_imagef(inPlane1, sampler, coord).r * depthScale;
  yuv.z  = read_imagef(inPlane2, sampler, coord).r * depthScale;

  return yuv_to_rgb(yuv, coef, lumaZero, chromaZero, blackLevel);
}

static float3 uyvy_to_rgb(constant  float*    coef,
                             float     lumaZero,
                             float     chromaZero,
                             float     blackLevel,
                             float     depthScale,
                             float2    coord,
                   read_only image2d_t inImage) {

  // Note the order: gbr instead of rgb. See https://www.opengl.org/registry/specs/APPLE/rgb_422.txt
  float3 yuv = read_imagef(inImage, sampler, coord).gbr;
  return yuv_to_rgb(yuv, coef, lumaZero, chromaZero, blackLevel);
}


